<?php

/**
 * @file
 * Non-admin page callbacks.
 */


/**
 * Callback function from hook_menu().
 * The URL will pass in parameters. The 1st should be the ISBN or ASIN. The 2nd should be whether it's an ISBN or ASIN.
 * The 3rd parameter is optional and will indicate if the details being asked for are from an eBooks list
 */
function booklists_detail($type, $code, $delta = NULL, $ebooks = NULL) {
  // Redirect to PAC details page if possible.
  if (module_exists('pac') && $type == 'ISBN') {
    $result = PolarisAPI::searchBibs('q=' . $code, 'keyword/ISBN'); // Get bib number for Polaris.
    $bib_no = $result->BibSearchRows[0]->ControlNumber;
    if (!empty($bib_no)) {
      drupal_goto('search/detail/' . $bib_no);
    }
  }
  else if (module_exists('pac') && $type == 'ASIN') {
    $upc = booklists_get_amazon_upc($code);
    $result = PolarisAPI::searchBibs('q=UPC=' . $upc, 'boolean'); // Try UPC first.
    $bib_no = $result->BibSearchRows[0]->ControlNumber;
    if (!empty($bib_no)) {
      drupal_goto('search/detail/' . $bib_no);
    }
    else {
      $ean = booklists_get_amazon_ean($code);
      $result = PolarisAPI::searchBibs('q=' . $ean, 'keyword/ISBN'); // Try EAN 2nd.
      $bib_no = $result->BibSearchRows[0]->ControlNumber;
      if (!empty($bib_no)) {
        drupal_goto('search/detail/' . $bib_no);
      }
      else {
        $isbn = booklists_get_amazon_isbn($code);
        $result = PolarisAPI::searchBibs('q=' . $isbn, 'keyword/ISBN'); // Try standard ISBN last.
        $bib_no = $result->BibSearchRows[0]->ControlNumber;
        if (!empty($bib_no)) {
          drupal_goto('search/detail/' . $bib_no);
        }
      }
    }
  }
  $item = array('type' => $type, 'code' => $code, 'delta' => $delta, 'ebooks' => $ebooks);
  $content = array();
  $content[] = array(
    '#type' => 'markup',
    '#markup' => theme('booklists_detail', $item),
  );
  return $content;
}

/**
 * Function to query Polaris via ODBC connection.
 *
 * @param str $list
 *   Which list is being displayed (coming soon, new arrivals, etc.)
 * @param bool $results_only
 *   Whether or not to show 
 * @param int $results_per_page
 *   The number of results per page.
 * @param int $page
 *   The page number to retrieve results for.  Defaults to the first page.
 */
function booklists_query_polaris($list, $results_only = FALSE, $results_per_page = 20, $page = 1) {
  // Sets the TDS Version environment variable.
  putenv('TDSVER=7.0');
  if (!empty($conf['polaris_mssql_config'])) {
    $mssql_config = $conf['polaris_mssql_config'];
  }
  else {
    $booklists_polaris_odbc_host = variable_get('booklists_polaris_odbc_host');
    $booklists_polaris_odbc_dbname = variable_get('booklists_polaris_odbc_dbname');
    $booklists_polaris_odbc_username = variable_get('booklists_polaris_odbc_username');
    $booklists_polaris_odbc_password = variable_get('booklists_polaris_odbc_password');
    $mssql_config = array(
      'user' => $booklists_polaris_odbc_username,
      'password' => $booklists_polaris_odbc_password,
      'dsn' => 'dblib:host=' . $booklists_polaris_odbc_host . ';dbname=' . $booklists_polaris_odbc_dbname,
    );
  }
  $query_params = drupal_get_query_parameters();

  // Modify the query based on specific list type selected by user.
  switch ($query_params['list_type']) {
    case 'movies': // DVD & blu-ray
      //$limiters = 'AND cir.AssignedCollectionID in (5,12,15,24,53,74,104,106)';
      $limiters = 'AND br.PrimaryMARCTOMID in (17,18,33,34,40)';
      break;
    case 'dvds':
      //$limiters = 'AND cir.AssignedCollectionID in (15,24,53,74,104,106)';
      $limiters = 'AND br.PrimaryMARCTOMID in (33)';
      break;
    case 'blu_ray':
      //$limiters = 'AND cir.AssignedCollectionID in (5,12,104,106)';
      $limiters = 'AND br.PrimaryMARCTOMID in (40)';
      break;
    case 'books': // LP & Regular
      //$limiters = 'AND cir.AssignedCollectionID in ()';
      $limiters = 'AND br.PrimaryMARCTOMID in (1,27,42,43)';
      break;
    case 'large_print':
      //$limiters = 'AND cir.AssignedCollectionID in (31,32,33,34,35,36,94)';
      $limiters = 'AND br.PrimaryMARCTOMID in (27)';
      break;
    case 'regular_print':
      //$limiters = 'AND cir.AssignedCollectionID in ()'; ???
      $limiters = 'AND br.PrimaryMARCTOMID in (1)'; // Book only. Not large print (27).
      break;
    case 'audiobooks':
      //$limiters = 'AND cir.AssignedCollectionID in ()';
      $limiters = 'AND br.PrimaryMARCTOMID in (22,37,41)';
      break;
    case 'ebooks':
      $limiters = 'AND br.PrimaryMARCTOMID in (36)';
      break;
    case 'music':
      $limiters = 'AND br.PrimaryMARCTOMID in (5,21,35)';
      break;
    case 'childrens':
      $limiters = 'AND cir.AssignedCollectionID in (3,6,11,12,13,14,15,16,17,18,19,20,21,22,23,67,69,70,71,72,73,105,106)';
      break;
    case 'teens':
      $limiters = 'AND cir.AssignedCollectionID in (79,80,81,82,83,84,85,86,103)';
      break;
    default:
      $limiters = '';
      break;
  }
  $availability = '';
  // Modify the result set if the user wants to show only available items.
  if (isset($query_params['available_only'])) {
    $availability = 'AND cir.ItemStatusID in (1)';
	}
  try {
    switch ($list) {
      case 'coming_soon':
        // Based on stored procedure "DASH_OnOrderItems".    
        $sql = "SELECT DISTINCT
        br.BibliographicRecordID,
        br.FirstAvailableDate,
        br.PrimaryMARCTOMID,
        br.BrowseAuthor,
        br.BrowseTitle,
        MAX(cir.ItemStatusDate) as ItemStatusDate
        FROM Polaris.Polaris.BibliographicRecords br WITH (NOLOCK) 
        INNER JOIN Polaris.Polaris.CircItemRecords cir WITH (NOLOCK)
        ON br.bibliographicrecordid = cir.AssociatedBibRecordID
        WHERE cir.ItemStatusID = 13
        AND br.DisplayInPAC = 1
        " . $limiters . "
        " . $availability . "
        GROUP BY br.bibliographicrecordid, br.browsetitle, br.browseauthor, br.FirstAvailableDate, br.PrimaryMARCTOMID
        ORDER BY ItemStatusDate DESC";
        /*
        OFFSET " . $offset . " ROWS
        FETCH NEXT " . $results_per_page . " ROWS ONLY*/
        break;
      case 'just_checked_in':
        // Custom. No dashboard exists for just checked in.
        $sql = "SELECT DISTINCT
        cir.AssociatedBibRecordID AS BibliographicRecordID,
        MAX(cir.LastCircTransactionDate) AS LastCircTransactionDate,
        cir.MaterialTypeID
        FROM Polaris.Polaris.BibliographicRecords br WITH (NOLOCK) 
        INNER JOIN Polaris.Polaris.CircItemRecords cir WITH (NOLOCK)
        ON br.bibliographicrecordid = cir.AssociatedBibRecordID
        WHERE cir.ItemStatusID = 1
        AND cir.DisplayInPAC = 1
        " . $limiters . "
        " . $availability . "
        AND cir.LastCircTransactionDate > getdate() - 14
        GROUP BY cir.AssociatedBibRecordID, cir.MaterialTypeID
        ORDER BY LastCircTransactionDate DESC";
        break;
      case 'most_popular':
        // Based on stored procedure "DASH_MostCircedTitles".
        $sql = "SELECT DISTINCT
        br.BibliographicRecordID,
        br.FirstAvailableDate,
        br.PrimaryMARCTOMID,
        br.BrowseAuthor,
        br.BrowseTitle,
        d.PeriodicCircCount
        FROM Polaris.Polaris.BibDashboardStatistics d WITH (NOLOCK)
        INNER JOIN Polaris.Polaris.BibliographicRecords br WITH (NOLOCK)
        ON br.BibliographicRecordID = d.BibliographicRecordID
        INNER JOIN Polaris.Polaris.CircItemRecords cir WITH (NOLOCK)
        ON br.bibliographicrecordid = cir.AssociatedBibRecordID
        WHERE br.DisplayInPAC = 1
        AND br.BrowseTitle NOT LIKE '%JUVENILE PAPERBACK%'
        " . $limiters . "
        " . $availability . "
        ORDER BY d.PeriodicCircCount DESC";
        break;
      case 'new_arrivals':
        // Based on stored procedure "DASH_NewReleases".
        $sql = "SELECT DISTINCT
          br.BibliographicRecordID,
          br.FirstAvailableDate,
          br.PrimaryMARCTOMID,
          br.BrowseAuthor,
          br.BrowseTitle
        FROM
          Polaris.Polaris.BibliographicRecords br with (nolock),
          Polaris.Polaris.CircItemRecords cir WITH (NOLOCK)
        WHERE
          br.FirstAvailableDate > getdate() - 14
        AND  cir.AssociatedBibRecordID = br.BibliographicRecordID
        AND cir.DisplayInPAC = 1
        AND cir.ItemStatusID in (1,2,4,5,6)
        " . $limiters . "
        " . $availability . "
        ORDER BY br.FirstAvailableDate DESC";
        break;
    }

    /**
     * Try to pull results from 24 hour cache first. If not available, create a PDO connection to SQL Server.
     */
    $cid = md5($sql);
    $cached = cache_get($cid, 'cache');
    if ($cached && time() < $cached->expire) {
      $bibs = $cached->data;
    }
    else {
      // Not in cache. Let's run the query.
      $conn = new PDO($mssql_config['dsn'], $mssql_config['user'], $mssql_config['password']);
      $conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
      // Prepare the SQL query and execute.
      try {
        $q = $conn->prepare($sql);
        $q->execute();
        // Grab the results.
        $bibs = $q->fetchAll(PDO::FETCH_ASSOC);
        if (!empty($availability)) {
          cache_set($cid, $bibs, 'cache', time() + 86400); // Cache the query result for 24 hours as long as it doesn't contain availability info. Those need to be done dynmaically.
        }
      }
      catch (Exception $e) {
        watchdog('Booklists', 'There was an issue executing SQL query (most likely a syntax error): @exception', array('@exception' => $sql), WATCHDOG_ERROR);
        return FALSE;
      }
      // Close the connection.
      $conn = NULL;
    }
    $total_items = count($bibs);
    $output_rows = '';
    $query = '';
    $page = pager_find_page();
    $offset = $page * $results_per_page;
    $maximum = $offset + $results_per_page;
    // Build the polaris query with all of these bib ids in it.
    foreach ($bibs as $key => $bib) {
      // Let's only query Polaris for the ones being shown on the currently-active page. Not ALL of them. Else we get an error.
      $l = $key + 1;
      if ($l > $offset && $l <= $maximum) {        
        $bib_id = $bib['BibliographicRecordID'];
        if ($key == $offset) {
          $query .= $bib_id;
        }
        else {
          $query .= ',' . $bib_id;
        }
      }
    }
    $query = 'q=CN=%7Blist%7D' . $query . '%7B/list%7D&bibsperpage=100';
    if ($results_only == FALSE) { // Page display
      // Load include required for bento_boxes_build_boolean_polaris_query().
      module_load_include('inc', 'bento_boxes', 'bento_boxes.pages');
      $query .= bento_boxes_build_boolean_polaris_query(100, 1);
      $query = str_replace('+NOT+TOM=ebk', '', $query);
      $query = str_replace('%7B/list%7D&bibsperpage=100', '%7B/list%7D', $query);
      if (empty($query_params['sort_by'])) {
        // Don't sort by relevance by default like bento boxes does. Let's not sort. That way the list will come back in the same order that we sent it in.
        $query = str_replace('+sortby+RELEVANCE', '', $query);
      }
    }
    $results = PolarisAPI::searchBibs($query, 'boolean');
    // Print out the details of each item.
    if ($results_only == FALSE) { // Page display.

      // START GETTING RATINGS FROM AMAZON.
      if (isset($query_params['sort_by']) && $query_params['sort_by'] == 'reviews') {
        foreach ($results->BibSearchRows as $key => $row) {
          if (!empty($row->ISBN)) {
            $asin = $row->ISBN;
          }
          if (empty($item) && !empty($row->UPC)) {
            $asin = amazon_upc_lookup($row->UPC); // Technically it's not an ISBN. It's an ASIN we're retrieving here, but let's just go with it for now.
            if (empty($asin) && strlen($code) == 13) {
              // If the UPC doesn't pull anything back from Amazon, try it as an EAN to see if we get anything.
              $asin = amazon_ean_lookup($code);
            }
          }
          $params = array(
            'ItemId' => $asin,
            'IdType' => $type,
            'ResponseGroup' => 'Reviews'
          );
          $results_amazon = amazon_http_request('ItemLookup', $params);
          $item = (array) $results_amazon->Items->Item->CustomerReviews->IFrameURL;
          $review = 0.05; // Basically make it 0. But theme_fivestar_static won't display it if it's absolute zero.
          if (!empty($item)) {
            $iframe_url = $item[0];
            $iframe_content = file_get_contents($iframe_url);
            preg_match('/alt="([^\s]*) out of 5 stars"/', $iframe_content, $matches);
            $review = $matches[1];
            if (empty($review)) { $review = 0.05; }
          }
          $results->BibSearchRows[$key]->review = $review;
        }
        // Sort based on ratings.
        usort($results->BibSearchRows, 'booklists_sort_reviews');
      }
      // END RATINGS.

      // OUTPUT
      foreach ($results->BibSearchRows as $key => $row) {
        $output_rows .= theme('bento_boxes_polaris_search_result', array('row' => $row));
      }
      if (count($results) == 0 || empty($results)) {
        $output_rows .= '<div class="view-empty"><p>' . t('No results were found.') . '</p></div>';
      }
      else {
        // Add pagers.
        pager_default_initialize($total_items, $results_per_page);
        $pager = theme('pager');
        $pager = str_replace('>next', '>Next', $pager);
        $pager = str_replace('previous<', 'Previous<', $pager);
        $output_rows = '<div class="bb_search-results-sort-paging">
          <div class="bb_search-results-sort">
            <div class="bb_search-results-sort-link" style="width: 80%">' . bento_boxes_build_sort_links() . '</div>
            <div class="bb_search-results-paging">' . $pager . '</div>
          </div>
        </div>' . $output_rows;
        // Bottom pager
        $output_rows .= '<div class="bb_search-results-sort-paging"><div class="bb_search-results-sort"> <div class="bb_search-results-paging">' . $pager . '</div></div></div>';
      }
    }
  }
  catch (Exception $e) {
    throw $e;
  }
  if ($results_only == FALSE) {
    return $output_rows;
  }
  else {
    return $results;
  }
}

/**
 * Custom sort function to put shipped items at the top, cancelled items at the bottom.
 */
function booklists_sort_reviews($a, $b) {
  if ($a->review > $b->review) {
    return -1; // A is greater than B. Move B down in the list.
  }
  else if ($a->review < $b->review) {
    return 1; // B is greater than A. Move it up in the list.
  }
  else {
    return 0; // Otherwise don't move it.
  }
}
